import 'package:contentsquare/contentsquare.dart';
import 'package:flutter/material.dart';

import '../../../../repository/manual_start_repository.dart';
import '../../../../widgets/app_bar.dart';

class ManualStartScreen extends StatefulWidget {
  const ManualStartScreen({
    Key? key,
  }) : super(key: key);

  @override
  State<ManualStartScreen> createState() => _ManualStartScreenState();
}

class _ManualStartScreenState extends State<ManualStartScreen> {
  bool? _hasManualStart;

  bool? get hasManualStart => _hasManualStart;

  set hasManualStart(bool? hasManualStart) {
    manualStartRepository.setIsManualStartEnabled(hasManualStart ?? false);
    _hasManualStart = hasManualStart;
  }

  final manualStartRepository = ManualStartRepository();
  
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: appBar(title: 'Manual Start'),
      body: Padding(
        padding: const EdgeInsets.symmetric(
          horizontal: 20.0,
          vertical: 50.0,
        ),
        child: Column(
          children: [
            Text(
              'When manual start is enabled, the contentsquare sdk will not  '
              'be initialized when the app start. Instead you will have to '
              'pressed the "Start SDK Manually" button to start the '
              'contentsquare sdk.',
              style: Theme.of(context).textTheme.subtitle2,
            ),
            SizedBox(height: 40),
            GestureDetector(
              onTap: () => hasManualStart == null
                  ? null
                  : setState(() => hasManualStart = !hasManualStart!),
              behavior: HitTestBehavior.opaque,
              child: Row(
                mainAxisSize: MainAxisSize.max,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Text('Enable manual start'),
                  SizedBox(width: 20),
                  FutureBuilder<bool>(
                    future: manualStartRepository.getIsManualStartEnabled(),
                    builder: (context, snapshot) {
                      if (!snapshot.hasData) {
                        return SizedBox.square(
                          dimension: 15,
                          child: CircularProgressIndicator(
                            color: Colors.black,
                            strokeWidth: 1,
                          ),
                        );
                      }

                      hasManualStart ??= snapshot.data;

                      return Switch(
                        activeColor: Colors.black,
                        value: hasManualStart!,
                        onChanged: (value) =>
                            setState(() => hasManualStart = value),
                      );
                    },
                  ),
                ],
              ),
            ),
            SizedBox(height: 20),
            Expanded(
              child: Align(
                alignment: Alignment.bottomCenter,
                child: OutlinedButton(
                  onPressed: () => _onManualStart(context),
                  child: Text('Start SDK Manually'),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Future<void> _onManualStart(BuildContext context) async {
    await Contentsquare().start();
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        padding: EdgeInsets.all(20),
        content: Text(
          'Contentsquare SDK started',
          textAlign: TextAlign.center,
        ),
      ),
    );
    Navigator.of(context).pop();
  }
}
